#include <stdio.h>
#include <math.h>
#include "ttdriver.h" 
#include "convfunc.h"

/* This is for conversion to get real temperatur units */
#define TC_A (.00112846386)
#define TC_B (.000234227257)
#define TC_C (8.733952111e-08)

float convert_temps( double V ) {
	double R, T, lR;

	if ( V == 10 ) return -1e9;
	R = 20000. * V / ( 10 - V );
	if ( R < 180 ) R = 180;
	else if ( R > 8e6 ) R = 8e6; /* Confine vals to table */
	lR = log(R);
	T = 1 / ( TC_A + TC_B * lR + TC_C * lR * lR * lR );
	T -= 273.16;
	return T;
}

/* for 0-12v divide on thermistor */
float convert_temps12v( double V ) {
	double R, T, lR;

	if ( V == 12 ) return -1e9;
	R = 20000. * V / ( 12 - V );
	if ( R < 180 ) R = 180;
	else if ( R > 8e6 ) R = 8e6; /* Confine vals to table */
	lR = log(R);
	T = 1 / ( TC_A + TC_B * lR + TC_C * lR * lR * lR );
	T -= 273.16;
	return T;
}

  
// converts Real units (like flow) to Voltage
// where R0 = 0 (so V0 is the voltage offset).
float convert_RtoV ( float Rx, float R1, float V1, float V0) {

	return V0 + Rx/R1 * (V1 - V0);

}

// converts Voltage to Real (like flow).
float convert_VtoR ( float Vx, float R1, float V1, float V0) {

	if ( Vx < 0 )
		Vx = -Vx;	
	else if ( Vx > 10 )
		Vx = 10;

	return R1 * (Vx - V0) / (V1 - V0);
}

// converts Voltage to DAC
int convert_VtoD ( float Vx, float Vmax ) {
	
	if ( Vx < 0 )
		Vx = 0;	
	else if ( Vx > 10 )
		Vx = 10;

	return	floor( Vx / Vmax * 4095 );
}

// converts DAC to Voltage
float convert_DtoV ( int DAC, float Vmax ) {
	float Vx;
	
	if ( DAC > 4095 )
		DAC = 4095;
	else if ( DAC < 0 )
		DAC = 0;

	Vx = Vmax * (float) DAC / 4095.0;

	return Vx;
}


// Functions below are Erik's original -- with variable name changes.
// this routine is a generalized convert to get the dac counts
int convert_to_dac ( float Rx, float R1, float Vmax, float V0 ) {

	int dac_value, dac_offset;

	if (Rx > R1)
		Rx = R1;
	else if (Rx < 0)
		Rx = 0;	

	dac_offset = 4095 * V0/Vmax;

// this prevents you from getting a number bigger than can be handled
// since the max dac value is 4095

	dac_value = (4095 - dac_offset) * Rx/R1;
	dac_value = dac_value + dac_offset;

//	printf(" the dac value is %i \n",dac_value);

	return dac_value;

}


// this routine is a generalized convert from ad voltages
// this routine assumes lineratity
float convert_from_ad (float Vx, float R1, float V1, float V0) {

	if ( Vx < 0 )
		Vx = -Vx;	
	else if ( Vx > 10 )
		Vx = 10;

	return R1 * (Vx - V0) / (V1 - V0);
}

